/*************************************************************************

 cave.macros.h

 Authors:	Carolina Cruz-Neira, Dave Pape
 Version:	2.6
 Last revised:	15 February 1997

  Archaic macros for getting tracking data or transformations

**************************************************************************/

/*
*	Copyright 1995,1997, Board of Trustees of the University of Illinois,
*	Electronic Visualization Laboratory.
*	All rights reserved.
*
*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF EVL.
*	Unless otherwise agreed to in writing by EVL, this code is
*	to be used and accessed only by SIGGRAPH VROOM participants.
*	It is not public domain software.  Call Margaret Rawlings at
*	(312)-996-3002 for details.
*
*/

#ifndef	__CAVE_MACROS_H__
#define	__CAVE_MACROS_H__

#include <cave.h>


#ifdef __cplusplus
extern "C" {
#endif


/*************************************************************************/
/** Old macros - do not use these any more; use CAVEGetPosition(),	**/
/**   CAVEGetOrientation(), CAVEGetVector(), CAVEHeadTransform(),	**/
/**   & CAVEWandTransform() instead					**/
/*************************************************************************/

#define  CAVEGetHead(xhead, yhead, zhead) \
   {                                   \
   xhead = CAVESensor1 -> x;              \
   yhead = CAVESensor1 -> y;              \
   zhead = CAVESensor1 -> z;               \
   }
#define  CAVEGetWand(xw,  yw,  zw) \
   { \
    xw = CAVESensor2 -> x; \
    yw = CAVESensor2 -> y; \
    zw = CAVESensor2 -> z; \
   }
    
#define  CAVEGetHeadOrientation(azimuth, elevation, wroll) \
   {                                   \
   azimuth = CAVESensor1 -> azim;              \
   elevation = CAVESensor1 -> elev;              \
   wroll = CAVESensor1 -> roll;               \
   }

#define  CAVEGetWandOrientation(azimuth, elevation, wroll) \
   { \
   azimuth = CAVESensor2 -> azim;              \
   elevation = CAVESensor2 -> elev;              \
   wroll = CAVESensor2 -> roll; \
   }


#ifdef OPENGL

#define  CAVEWandOrientation  \
   { \
   glTranslatef( CAVESensor2 -> x,  CAVESensor2 -> y, CAVESensor2 -> z); \
   glRotatef(CAVESensor2->azim, 0.0f,1.0f,0.0f); \
   glRotatef(CAVESensor2->elev, 1.0f,0.0f,0.0f); \
   glRotatef(CAVESensor2->roll, 0.0f,0.0f,1.0f); \
   glTranslatef( -CAVESensor2 -> x,  -CAVESensor2 -> y, -CAVESensor2 -> z); \
   }
   
#define  CAVEHeadOrientation  \
   { \
   glTranslatef( CAVESensor1 -> x,  CAVESensor1 -> y, CAVESensor1 -> z); \
   glRotatef(CAVESensor1->azim, 0.0f,1.0f,0.0f); \
   glRotatef(CAVESensor1->elev, 1.0f,0.0f,0.0f); \
   glRotatef(CAVESensor1->roll, 0.0f,0.0f,1.0f); \
   glTranslatef( -CAVESensor1 -> x, -CAVESensor1 -> y, -CAVESensor1 -> z); \
   }

#else

#define  CAVEWandOrientation  \
   { \
   translate( CAVESensor2 -> x,  CAVESensor2 -> y, CAVESensor2 -> z); \
   rot(CAVESensor2->azim ,'y'); \
   rot(CAVESensor2->elev, 'x'); \
   rot(CAVESensor2->roll, 'z'); \
   translate( -CAVESensor2 -> x,  -CAVESensor2 -> y, -CAVESensor2 -> z); \
   }
   
#define  CAVEHeadOrientation  \
   { \
   translate( CAVESensor1 -> x,  CAVESensor1 -> y, CAVESensor1 -> z); \
   rot(CAVESensor1->azim ,'y'); \
   rot(CAVESensor1->elev, 'x'); \
   rot(CAVESensor1->roll, 'z'); \
   translate( -CAVESensor1 -> x, -CAVESensor1 -> y, -CAVESensor1 -> z); \
   }

#endif


/* DTOR is single precision for speed */
#define DTOR(r) ((r)*0.01745329f)


#ifndef __sgi

#define cosf(x)		cos(x)
#define sinf(x)		sin(x)
#define sqrtf(x)	sqrt(x)
#define atan2f(x,y)	atan2(x,y)
#define sginap(t)	usleep(t*10000)

#endif


/*************************************/
/* Wand unit direction vector macros */
/*************************************/

#define CAVEGetWandFront(hx, hy, hz)    \
{   \
 hx = -cosf(DTOR(CAVESensor2->elev)) * sinf(DTOR(CAVESensor2->azim));	\
 hy = sinf(DTOR(CAVESensor2->elev));					\
 hz = -cosf(DTOR(CAVESensor2->elev)) * cosf(DTOR(CAVESensor2->azim));	\
}

#define CAVEGetWandUp(hx, hy, hz)    \
{   \
 hx = -cosf(DTOR(CAVESensor2->azim)) * sinf(DTOR(CAVESensor2->roll)) +	\
	sinf(DTOR(CAVESensor2->azim)) * sinf(DTOR(CAVESensor2->elev)) *	\
	cosf(DTOR(CAVESensor2->roll));					\
 hy = cosf(DTOR(CAVESensor2->roll)) * cosf(DTOR(CAVESensor2->elev));	\
 hz = sinf(DTOR(CAVESensor2->azim)) * sinf(DTOR(CAVESensor2->roll)) +	\
	cosf(DTOR(CAVESensor2->azim)) * sinf(DTOR(CAVESensor2->elev)) *	\
	cosf(DTOR(CAVESensor2->roll));					\
}

#define CAVEGetWandLeft(hx, hy, hz)    \
{   \
 hx = -cosf(DTOR(CAVESensor2->azim)) * cosf(DTOR(CAVESensor2->roll)) -	\
	sinf(DTOR(CAVESensor2->azim)) * sinf(DTOR(CAVESensor2->elev)) *	\
	sinf(DTOR(CAVESensor2->roll));					\
 hy = -cosf(DTOR(CAVESensor2->elev)) * sinf(DTOR(CAVESensor2->roll));	\
 hz = sinf(DTOR(CAVESensor2->azim)) * cosf(DTOR(CAVESensor2->roll)) -	\
	cosf(DTOR(CAVESensor2->azim)) * sinf(DTOR(CAVESensor2->elev)) *	\
	sinf(DTOR(CAVESensor2->roll));					\
}

/*************************************/
/* Head unit direction vector macros */
/*************************************/

#define CAVEGetHeadFront(hx, hy, hz)    \
{   \
 hx = -cosf(DTOR(CAVESensor1->elev)) * sinf(DTOR(CAVESensor1->azim));	\
 hy = sinf(DTOR(CAVESensor1->elev));					\
 hz = -cosf(DTOR(CAVESensor1->elev)) * cosf(DTOR(CAVESensor1->azim));	\
}

#define CAVEGetHeadUp(hx, hy, hz)    \
{   \
 hx = -cosf(DTOR(CAVESensor1->azim)) * sinf(DTOR(CAVESensor1->roll)) +	\
	sinf(DTOR(CAVESensor1->azim)) * sinf(DTOR(CAVESensor1->elev)) *	\
	cosf(DTOR(CAVESensor1->roll));					\
 hy = cosf(DTOR(CAVESensor1->roll)) * cosf(DTOR(CAVESensor1->elev));	\
 hz = sinf(DTOR(CAVESensor1->azim)) * sinf(DTOR(CAVESensor1->roll)) +	\
	cosf(DTOR(CAVESensor1->azim)) * sinf(DTOR(CAVESensor1->elev)) *	\
	cosf(DTOR(CAVESensor1->roll));					\
}

#define CAVEGetHeadLeft(hx, hy, hz)    \
{   \
 hx = -cosf(DTOR(CAVESensor1->azim)) * cosf(DTOR(CAVESensor1->roll)) -	\
	sinf(DTOR(CAVESensor1->azim)) * sinf(DTOR(CAVESensor1->elev)) *	\
	sinf(DTOR(CAVESensor1->roll));					\
 hy = -cosf(DTOR(CAVESensor1->elev)) * sinf(DTOR(CAVESensor1->roll));	\
 hz = sinf(DTOR(CAVESensor1->azim)) * cosf(DTOR(CAVESensor1->roll)) -	\
	cosf(DTOR(CAVESensor1->azim)) * sinf(DTOR(CAVESensor1->elev)) *	\
	sinf(DTOR(CAVESensor1->roll));					\
}


#ifdef __cplusplus
}
#endif

#endif
